﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_test_transformations
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void DrawSmiley(Graphics gr, RectangleF bounds)
        {
            // Outline.
            gr.DrawRectangle(Pens.Green, bounds);

            float thickness = bounds.Width / 30;
            bounds.Inflate(-thickness, -thickness);
            using (Pen pen = new Pen(Color.Red, thickness))
            {
                // Face.
                gr.FillEllipse(Brushes.Yellow, bounds);
                gr.DrawEllipse(pen, bounds);
                pen.Color = Color.Black;

                // Smile.
                RectangleF rect = bounds;
                rect.Inflate(
                    -bounds.Width / 6,
                    -bounds.Width / 6);
                gr.DrawArc(pen, rect, 0, 180);

                // Nose.
                pen.Width /= 2f;
                float cx = bounds.X + bounds.Width * 0.5f;
                float cy = bounds.Y + bounds.Height * 0.55f;
                float nose_width = bounds.Width * 0.2f;
                float nose_height = bounds.Width * 0.35f;
                RectangleF nose_rect = new RectangleF(
                    cx - nose_width / 2f,
                    cy - nose_height / 2f,
                    nose_width,
                    nose_height);
                gr.FillEllipse(Brushes.LightBlue, nose_rect);
                pen.Color = Color.Blue;
                gr.DrawEllipse(pen, nose_rect);

                // Left eye.
                cx = bounds.X + bounds.Width * 0.3f;
                cy = bounds.Y + bounds.Height * 0.3f;
                float eye_width = nose_width * 0.85f;
                float eye_height = nose_height * 0.75f;
                RectangleF eye_rect = new RectangleF(
                    cx - eye_width / 2f,
                    cy - eye_height / 2f,
                    eye_width,
                    eye_height);
                gr.FillEllipse(Brushes.White, eye_rect);
                pen.Color = Color.Black;
                gr.DrawEllipse(pen, eye_rect);

                // Left pupil.
                float px = cx + nose_width / 4f;
                float py = cy;
                float pupil_width = nose_width / 2f;
                float pupil_height = nose_height / 2f;
                RectangleF pupil_rect = new RectangleF(
                    px - pupil_width / 2f,
                    py - pupil_height / 2f,
                    pupil_width,
                    pupil_height);
                gr.FillEllipse(Brushes.Black, pupil_rect);

                // Right eye.
                cx = bounds.X + bounds.Width * 0.7f;
                cy = bounds.Y + bounds.Height * 0.3f;
                eye_width = nose_width * 0.85f;
                eye_height = nose_height * 0.75f;
                eye_rect = new RectangleF(
                    cx - eye_width / 2f,
                    cy - eye_height / 2f,
                    eye_width,
                    eye_height);
                gr.FillEllipse(Brushes.White, eye_rect);
                pen.Color = Color.Black;
                gr.DrawEllipse(pen, eye_rect);

                // Right pupil.
                px = cx + nose_width / 4f;
                py = cy;
                pupil_width = nose_width / 2f;
                pupil_height = nose_height / 2f;
                pupil_rect = new RectangleF(
                    px - pupil_width / 2f,
                    py - pupil_height / 2f,
                    pupil_width,
                    pupil_height);
                gr.FillEllipse(Brushes.Black, pupil_rect);
            }
        }

        private void DrawPicture(PictureBox pic, Graphics gr)
        {
            gr.Clear(pic.BackColor);
            gr.SmoothingMode = SmoothingMode.AntiAlias;
            DrawSmiley(gr, pic.ClientRectangle);
        }

        private void picOriginal_Paint(object sender, PaintEventArgs e)
        {
            DrawPicture(sender as PictureBox, e.Graphics);
        }

        private void picTranslate_Paint(object sender, PaintEventArgs e)
        {
            float dx, dy;
            float.TryParse(txtDx.Text, out dx);
            float.TryParse(txtDy.Text, out dy);
            e.Graphics.TranslateTransform(dx, dy);
            DrawPicture(sender as PictureBox, e.Graphics);
        }

        private void picRotate_Paint(object sender, PaintEventArgs e)
        {
            float angle;
            float.TryParse(txtRotate.Text, out angle);
            e.Graphics.RotateTransform(angle);
            DrawPicture(sender as PictureBox, e.Graphics);
        }

        private void picScale_Paint(object sender, PaintEventArgs e)
        {
            float sx, sy;
            float.TryParse(txtSx.Text, out sx);
            float.TryParse(txtSy.Text, out sy);
            try
            {
                e.Graphics.ScaleTransform(sx, sy);
                DrawPicture(sender as PictureBox, e.Graphics);
            }
            catch { }
        }

        private void picSkew_Paint(object sender, PaintEventArgs e)
        {
            float sx, sy;
            float.TryParse(txtSkewX.Text, out sx);
            float.TryParse(txtSkewY.Text, out sy);

            Matrix matrix = new Matrix();
            matrix.Shear(sx, sy);

            e.Graphics.Transform = matrix;

            DrawPicture(sender as PictureBox, e.Graphics);
        }

        private void picElements_Paint(object sender, PaintEventArgs e)
        {
            float sx = 1;
            float sky = 0;
            float skx = 0;
            float sy = 1;
            float dx = 0;
            float dy = 0;
            float.TryParse(txtM11.Text, out sx);
            float.TryParse(txtM12.Text, out sky);
            float.TryParse(txtM21.Text, out skx);
            float.TryParse(txtM22.Text, out sy);
            float.TryParse(txtMdx.Text, out dx);
            float.TryParse(txtMdy.Text, out dy);

            try
            {
                Matrix matrix = new Matrix(sx, sky, skx, sy, dx, dy);
                e.Graphics.Transform = matrix;
                DrawPicture(sender as PictureBox, e.Graphics);
            }
            catch { }
        }

        private void txtTranslate_TextChanged(object sender, EventArgs e)
        {
            picTranslate.Refresh();
        }

        private void txtRotate_TextChanged(object sender, EventArgs e)
        {
            picRotate.Refresh();
        }

        private void txtScale_TextChanged(object sender, EventArgs e)
        {
            picScale.Refresh();
        }

        private void txtSkew_TextChanged(object sender, EventArgs e)
        {
            picSkew.Refresh();
        }

        private void txtElements_TextChanged(object sender, EventArgs e)
        {
            picElements.Refresh();
        }
    }
}
